% function [curve, solution] = Exp (mu, v, manifold)
%   Compute the exponential map of 'v' expressed in the tangent space at 'mu'.
%
%   Input:
%     mu:       the origin of the tangent space. This should be a D-vector.
%     v:        a D-vector expressed in the tangent space at 'mu'. This vector
%               is often a linear combination of points produced by the
%               'compute_geodesic' function.
%     manifold: a structure describing how the metric tensor changes. This is
%               created with the 'create_manifold' function.
%
%   Output:
%     curve:    
%     solution: the output structure from the ODE solver. This is mostly used
%               for debugging.
%
%   See also:
%     compute_geodesic, create_manifold, ode45, myode45

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
function [curve, solution] = Exp (mu, v, manifold)
  mu = mu (:);
  v = v (:);
  dest_len = norm (v);

  %% Set up ODE system
  odefun = @(x, y) manifold.geo_fun (y, manifold);
 
  [curve, ~, solution] = ode_solve (odefun, mu, v, manifold);
end % function

function y = parzen_system_wrapper (y, manifold)
  D = numel (y) / 2;
  c = y (1:D);
  cm = y (D+1:end);
  f = parzen_system (c, cm, manifold);
  y = [cm; f];
end % function

function [c, len, solution] = ode_solve (odefun, mu, v, manifold)
  init = [mu(:); v(:)];
  t = [0, 1];
%  solution = ode45 (odefun, t, init); %, manifold);
  solution = myode45 (odefun, t, init, manifold);
  c = solution.y (1:numel(mu), :);
  len = curve_length (c, manifold);
end % function


