% function [curve, len, logarithm, failed, solution] = compute_geodesic (p1, p2, manifold, solution)
%   Compute the geodesic between two points.
%
%   Input:
%     p1:       a D-vector
%     p1:       a D-vector.
%     manifold: a structure describing how the metric tensor changes. This is
%               created with the 'create_manifold' function.
%
%   Output:
%     curve:     the geodesic.
%     len:       the length of the geodesic.
%     logarithm: D-vector containing the log map.
%     failed:    a boolean indicating if the numerical computation converged.
%     solution:  the output structures from the ODE solver. This is mostly
%                used for debugging.
%
%   See also:
%     compute_all_geodesics, create_manifold

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [curve, len, logarithm, failed, solution] = compute_geodesic (p1, p2, manifold, solution)
  if (strcmpi (manifold.weight_scheme, 'parzen'))
    [curve, len, logarithm] = compute_geodesic_parzen (p1, p2, manifold);
    failed = false;
    solution = [];
    return;
  end % if

  warning ('OFF', 'MATLAB:illConditionedMatrix');

  p1 = p1 (:);
  p2 = p2 (:);
  D = numel (p1);
  
  %% Set up boundary conditions
  bc = @(a, b) boundary (a, b, p1, p2);
  
  %% Set up ODE system
  options = manifold.geo_options;
  odefun = @(x, y) manifold.geo_fun (y, manifold);
  if (isfield (manifold, 'jacobian'))
    options = bvpset (options, 'FJacobian', @(x, y) manifold.jacobian (y, manifold));
  end % if
  
  %% Initial guess
  if (nargin < 4)
    x_init = [0, 1];
    line_init = p1;
    deriv_init = (p2 - p1);
    y_init = [line_init(:); deriv_init(:)];
    init = bvpinit (x_init, y_init);
  else
    init = solution;
  end % if
  
  %% Solve system
  warning ('error', 'MATLAB:bvp4c:RelTolNotMet');
  warning ('error', 'MATLAB:bvp5c:RelTolNotMet');
  try
    failed = false;
    solution = bvp5c (odefun, bc, init, options);
  catch
    failed = true;
  end
  if (failed) % The ODE solver seems to have diverged, so we try another solver
    try
      failed = false;
      solution = bvp4c (odefun, bc, init, options);
    catch
      failed = true;
    end
  end % if
  
  %% Provide the output
  if (failed)
    curve = [p1, p2];
    len = curve_length (curve, manifold);
    logarithm = (p2 - p1);
    logarithm = len * logarithm / norm (logarithm);
    solution = [];
  else
    curve = solution.y (1:D, :);
    logarithm = solution.y  ((D+1):end, 1);
    len = curve_length (curve, manifold);
  end % if
  logarithm = len * logarithm / norm (logarithm);
  
end % function

function bc = boundary (p1, p2, p1_goal, p2_goal)
  D = numel (p1_goal);
  d1 = p1 (1:D) - p1_goal (:);
  d2 = p2 (1:D) - p2_goal (:);
  bc = [d1; d2];
end % function


