% function manifold = create_manifold (mu, M, weight_scheme, lambda, varargin)
%   Combine metric tensors to form a manifold structure.
%
%   Input:
%     mu:            a cell array containing vectors describing the
%                    location of the metric tensors.
%     M:             a cell array containing the base metric tensors.
%     weight_scheme: a string indicating how the metric tensors should be
%                    combined. Possible values: 'local Gaussian' (default),
%                    'global Gaussian', 'diagonal'.
%     lambda:        a smoothing parameter describing how fast the metric
%                    tensor changes. Default: 1.
%
%   Output:
%     manifold: a structure describing the manifold. This is used as input
%               for the functions doing the actual computations.
%
%   See also:
%     ml_mean, compute_geodesic, compute_all_geodesic

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function manifold = create_manifold (mu, M, weight_scheme, lambda, varargin)
  %% Default arguments
  if (nargin < 3)
    weight_scheme = 'local Gaussian';
  end % if
  if (nargin < 4)
    lambda = 1;
  end % if
  weight_scheme = lower (weight_scheme);

  %% Ensure mean vectors are on the right form
  for r = 1:numel (mu)
    mu {r} = mu {r} (:);
  end % for
  
  %% Basic options for computing geodesics
  geo_options = bvpset ('NMax', 500);
  
  %% Basic manifold structure
  manifold = struct ();
  manifold.jacobian = @numerical_jacobian; % default; some manifolds overwrite this
  
  %% Different weight schemes => different actions
  switch (weight_scheme)
    case 'local gaussian'
      C = cellfun (@safe_chol, M, 'UniformOutput', false);
      geo_fun = @geodesic_system_local_gaussian;
      
    case 'diagonal'
      %% Ensure local metrics are on the right form
      for r = 1:numel (M)
        if (size (M{r}, 1) == size (M{r}, 2))
          M {r} = diag (M {r});
        end % if
        M {r} = M {r} (:);
      end % for
      C = cellfun (@sqrt, M, 'UniformOutput', false);
      geo_fun = @local_diagonal_gaussian_geodesic_system;
      geo_options = bvpset (geo_options, 'Vectorized', 'on');
      
    case 'global gaussian'
      manifold = struct (varargin{:});
      C = cellfun (@safe_chol, M, 'UniformOutput', false);
      geo_fun = @geodesic_system_global_gaussian;
      if (~isfield (manifold, 'Gamma'))
        manifold.Gamma = eye (numel (mu {1}));
      end % if
      
    otherwise
      error ('rml:unknown_weight_scheme', 'create_manifold: unknown weight scheme');
  end % switch

  %% Create manifold structure
  manifold.mu = mu;
  manifold.M = M;
  manifold.C = C;
  manifold.weight_scheme = weight_scheme;
  manifold.lambda = lambda;
  manifold.geo_fun = geo_fun;
  manifold.geo_options = geo_options;
end % function

function C = safe_chol (M)
  try
    C = chol (M);
  catch
    [V, lambda] = eig (M);
    C = V * sqrt (lambda) * V.';
  end % try_catch
end % function
